@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -85,7 +85,7 @@
  *  - transforming the values of the hymap:
  *    `auto dst_hymap = tuple_util::transform(change_value_functor, src_hymap);`
  *  - making a map:
- *    `auto a_map = tuple_util::make<hymap::keys<a, b>::values>('a', 42)`
+ *    `auto a_map = hymap::keys<a, b>::values('a', 42)`
  *  - converting to a map:
  *    `auto a_map = tuple_util::convert_to<hymap::keys<a, b>::values>(a_tuple_with_values)`
  */
@@ -96,10 +96,10 @@
 #define GT_COMMON_HYMAP_HPP_
 
 #include <type_traits>
+#include <utility>
 
 #include "../meta.hpp"
 #include "defs.hpp"
-#include "generic_metafunctions/utility.hpp"
 #include "host_device.hpp"
 #include "integral_constant.hpp"
 #include "tuple.hpp"
@@ -120,10 +120,10 @@ namespace gridtools {
         not_provided hymap_get_keys(...);
 
         template <class T, class Res = decltype(hymap_get_keys(std::declval<T const &>()))>
-        std::enable_if_t<!std::is_same<Res, not_provided>::value, Res> get_keys_fun(T const &);
+        std::enable_if_t<!std::is_same_v<Res, not_provided>, Res> get_keys_fun(T const &);
 
         template <class T, class Res = decltype(hymap_get_keys(std::declval<T const &>()))>
-        std::enable_if_t<std::is_same<Res, not_provided>::value, default_keys<T>> get_keys_fun(T const &);
+        std::enable_if_t<std::is_same_v<Res, not_provided>, default_keys<T>> get_keys_fun(T const &);
 
         template <class T>
         using get_keys = decltype(::gridtools::hymap_impl_::get_keys_fun(std::declval<T const &>()));
@@ -133,8 +133,7 @@ namespace gridtools {
 
         template <template <class...> class L, class... Values>
         struct values_are_nested_in_keys<L<Values...>,
-            std::enable_if_t<
-                std::is_same<L<Values...>, typename get_keys<L<Values...>>::template values<Values...>>::value>>
+            std::enable_if_t<std::is_same_v<L<Values...>, typename get_keys<L<Values...>>::template values<Values...>>>>
             : std::true_type {};
 
         template <template <class...> class Ctor>
@@ -149,18 +148,44 @@ namespace gridtools {
         not_provided hymap_from_keys_values(...);
 
         template <class T, class Res = decltype(hymap_from_keys_values(std::declval<T const &>()))>
-        std::enable_if_t<!std::is_same<Res, not_provided>::value, Res> get_from_keys_values_fun(T const &);
+        std::enable_if_t<!std::is_same_v<Res, not_provided>, Res> get_from_keys_values_fun(T const &);
 
         template <class T, class Res = decltype(hymap_from_keys_values(std::declval<T const &>()))>
-        std::enable_if_t<std::is_same<Res, not_provided>::value && values_are_nested_in_keys<T>::value,
+        std::enable_if_t<std::is_same_v<Res, not_provided> && values_are_nested_in_keys<T>::value,
             default_from_keys_values<T>>
         get_from_keys_values_fun(T const &);
 
         template <class T>
         using get_from_keys_values =
             decltype(::gridtools::hymap_impl_::get_from_keys_values_fun(std::declval<T const &>()));
+
+        template <class Key, class Map>
+        using element_at = tuple_util::element<meta::st_position<get_keys<Map>, Key>::value, Map>;
+
+        template <class T, class Keys = get_keys<T>>
+        struct keys_are_legit_sfinae
+            : std::bool_constant<meta::is_set<Keys>::value && meta::length<Keys>::value == tuple_util::size<T>::value> {
+        };
+
+        template <class, class = void>
+        struct keys_are_legit : std::false_type {};
+
+        template <class T>
+        struct keys_are_legit<T, std::enable_if_t<keys_are_legit_sfinae<T>::value>> : std::true_type {};
+
     } // namespace hymap_impl_
 
+    template <class T>
+    using is_hymap = std::bool_constant<is_tuple_like<T>::value && hymap_impl_::keys_are_legit<T>::value>;
+
+#ifdef __cpp_concepts
+    namespace concepts {
+        template <class T>
+        concept hymap = is_hymap<T>::value;
+    }
+#endif
+
+    using hymap_impl_::element_at;
     using hymap_impl_::get_from_keys_values;
     using hymap_impl_::get_keys;
 
@@ -168,19 +193,70 @@ namespace gridtools {
     using has_key = meta::st_contains<hymap_impl_::get_keys<Map>, Key>;
 
     namespace hymap {
-        template <class... Keys>
+        template <class...>
         struct keys {
-            template <class... Vals>
-            struct values {
-                static_assert(sizeof...(Vals) == sizeof...(Keys), "invalid hymap");
+            template <class...>
+            struct values;
 
-                tuple<Vals...> m_vals;
+#if !defined(__NVCC__) && defined(__clang__) && __clang_major__ <= 16
+            template <class... Vs>
+            values(Vs const &...) -> values<Vs...>;
+#endif
 
-                GT_TUPLE_UTIL_FORWARD_CTORS_TO_MEMBER(values, m_vals);
-                GT_TUPLE_UTIL_FORWARD_GETTER_TO_MEMBER(values, m_vals);
+            // NVCC 11 fails to do class template deduction in the case of nested templates
+            template <class... Args>
+            static constexpr GT_FUNCTION values<Args...> make_values(Args const &...args) {
+                return {args...};
+            }
+        };
 
-                friend keys hymap_get_keys(values const &) { return {}; }
-            };
+        template <class... Keys>
+        template <class... Vals>
+        struct keys<Keys...>::values {
+            static_assert(sizeof...(Vals) == sizeof...(Keys), "invalid hymap");
+
+            tuple<Vals...> m_vals;
+
+            template <class... Args,
+                std::enable_if_t<std::conjunction_v<std::is_constructible<Vals, Args>...>, int> = 0>
+            constexpr GT_FUNCTION values(Args &&...args) noexcept : m_vals{std::forward<Args>(args)...} {}
+
+            constexpr GT_FUNCTION values(Vals const &...args) noexcept : m_vals(args...) {}
+
+            constexpr GT_FUNCTION values(tuple<Vals...> &&args) noexcept : m_vals(std::move(args)) {}
+            constexpr GT_FUNCTION values(tuple<Vals...> const &args) noexcept : m_vals(args) {}
+
+            values() = default;
+            values(values const &) = default;
+            values(values &&) = default;
+            values &operator=(values const &) = default;
+            values &operator=(values &&) = default;
+
+            template <class Src>
+            constexpr GT_FUNCTION
+                std::enable_if_t<((!std::is_same_v<values, std::decay_t<Src>> && is_hymap<std::decay_t<Src>>::value) &&
+                                     ... &&
+                                     std::is_assignable_v<Vals &,
+                                         std::add_lvalue_reference_t<element_at<Keys, std::remove_reference_t<Src>>>>),
+                    values &>
+                operator=(Src &&src) {
+                (...,
+                    (tuple_util::host_device::get<meta::st_position<meta::list<Keys...>, Keys>::value>(m_vals) =
+                            tuple_util::host_device::get<meta::st_position<get_keys<std::decay_t<Src>>, Keys>::value>(
+                                src)));
+                return *this;
+            }
+
+            GT_TUPLE_UTIL_FORWARD_GETTER_TO_MEMBER(values, m_vals);
+
+            friend keys hymap_get_keys(values const &) { return {}; }
+        };
+
+        template <>
+        template <>
+        struct keys<>::values<> {
+            friend values tuple_getter(values const &) { return {}; }
+            friend keys hymap_get_keys(values const &) { return {}; }
         };
 
         template <class HyMap>
@@ -191,10 +267,11 @@ namespace gridtools {
 
         template <class MetaMap,
             template <class...> class KeyCtor = keys,
-            class KeysAndValues = meta::transpose<MetaMap>>
+            class KeysAndValues =
+                meta::if_<meta::is_empty<MetaMap>, meta::list<meta::list<>, meta::list<>>, meta::transpose<MetaMap>>>
         using from_meta_map = from_keys_values<meta::first<KeysAndValues>, meta::second<KeysAndValues>, KeyCtor>;
 
-        namespace hymap_impl_ {
+        namespace impl_ {
             template <class Maps>
             using merged_keys = meta::dedup<meta::transform<meta::first, meta::flatten<Maps>>>;
 
@@ -210,7 +287,7 @@ namespace gridtools {
             template <class Maps>
             struct merged_value_f {
                 template <class Key>
-                using apply = meta::lfold<get_first_folder, void, meta::transform<find_f<Key>::template apply, Maps>>;
+                using apply = meta::foldl<get_first_folder, void, meta::transform<find_f<Key>::template apply, Maps>>;
             };
 
             template <class Src>
@@ -227,7 +304,7 @@ namespace gridtools {
                 template <class Values, class Maps, class Keys = meta::flatten<meta::transform<get_keys, Maps>>>
                 using apply = typename get_from_keys_values<meta::first<Maps>>::template apply<Keys, Values>;
             };
-        } // namespace hymap_impl_
+        } // namespace impl_
     }     // namespace hymap
 } // namespace gridtools
 
@@ -245,8 +322,8 @@ namespace gridtools {
             class Decayed = std::decay_t<Map>,
             class I = meta::st_position<get_keys<Decayed>, Key>,
             std::enable_if_t<I::value != tuple_util::size<Decayed>::value, int> = 0>
-        GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) at_key(Map && map) noexcept {
-            return tuple_util::GT_TARGET_NAMESPACE_NAME::get<I::value>(wstd::forward<Map>(map));
+        GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) at_key(Map && map) noexcept {
+            return tuple_util::GT_TARGET_NAMESPACE_NAME::get<I::value>(std::forward<Map>(map));
         }
 
         template <class Key,
@@ -264,8 +341,8 @@ namespace gridtools {
             class Decayed = std::decay_t<Map>,
             class I = meta::st_position<get_keys<Decayed>, Key>,
             std::enable_if_t<I::value != tuple_util::size<Decayed>::value, int> = 0>
-        GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) at_key_with_default(Map && map) noexcept {
-            return tuple_util::GT_TARGET_NAMESPACE_NAME::get<I::value>(wstd::forward<Map>(map));
+        GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) at_key_with_default(Map && map) noexcept {
+            return tuple_util::GT_TARGET_NAMESPACE_NAME::get<I::value>(std::forward<Map>(map));
         }
 
         template <class Key,
@@ -274,7 +351,7 @@ namespace gridtools {
             class Decayed = std::decay_t<Map>,
             class I = meta::st_position<get_keys<Decayed>, Key>,
             std::enable_if_t<I::value == tuple_util::size<Decayed>::value, int> = 0>
-        GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR Default at_key_with_default(Map &&) noexcept {
+        GT_TARGET GT_FORCE_INLINE constexpr Default at_key_with_default(Map &&) noexcept {
             return {};
         }
     }
@@ -286,40 +363,57 @@ namespace gridtools {
                 struct adapter_f {
                     Fun m_fun;
                     template <size_t I, class Value, class Key = meta::at_c<Keys, I>>
-                    GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) operator()(Value &&value) const {
-                        return m_fun.template operator()<Key>(wstd::forward<Value>(value));
+                    GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) operator()(Value &&value) const {
+                        return m_fun.template operator()<Key>(std::forward<Value>(value));
+                    }
+                };
+
+                template <class Key>
+                struct at_generator_f {
+                    template <class Value>
+                    GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) operator()(Value &&value) const {
+                        return at_key<Key>(std::forward<Value>(value));
                     }
                 };
             } // namespace hymap_detail
 
             template <class Fun, class Map>
-            GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR auto transform(Fun && fun, Map && map) {
+            GT_TARGET GT_FORCE_INLINE constexpr auto transform(Fun && fun, Map && map) {
                 return tuple_util::GT_TARGET_NAMESPACE_NAME::transform_index(
-                    hymap_detail::adapter_f<Fun, get_keys<std::decay_t<Map>>>{wstd::forward<Fun>(fun)},
-                    wstd::forward<Map>(map));
+                    hymap_detail::adapter_f<Fun, get_keys<std::decay_t<Map>>>{std::forward<Fun>(fun)},
+                    std::forward<Map>(map));
             }
 
             template <class Fun, class Map>
-            GT_TARGET GT_FORCE_INLINE void for_each(Fun && fun, Map && map) {
+            GT_TARGET GT_FORCE_INLINE constexpr void for_each(Fun && fun, Map && map) {
                 tuple_util::GT_TARGET_NAMESPACE_NAME::for_each_index(
-                    hymap_detail::adapter_f<Fun, get_keys<std::decay_t<Map>>>{wstd::forward<Fun>(fun)},
-                    wstd::forward<Map>(map));
+                    hymap_detail::adapter_f<Fun, get_keys<std::decay_t<Map>>>{std::forward<Fun>(fun)},
+                    std::forward<Map>(map));
             }
 
             // Concatenate several maps into one. The type of the result is infered from the type of the first map.
             // Precondition: keys should not overlap.
             template <class... Maps>
-            GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR auto concat(Maps... maps) {
+            GT_TARGET GT_FORCE_INLINE constexpr auto concat(Maps... maps) {
                 static_assert(meta::is_set_fast<meta::concat<get_keys<Maps>...>>::value, GT_INTERNAL_ERROR);
-                return tuple_util::concat_ex<hymap_impl_::concat_result_maker_f>(wstd::move(maps)...);
+                return tuple_util::concat_ex<impl_::concat_result_maker_f>(std::move(maps)...);
             }
 
+            GT_TARGET GT_FORCE_INLINE constexpr keys<>::values<> concat() { return {}; }
+
             template <template <class...> class KeyCtor,
                 class Keys,
                 class Tup,
                 class HyMapKeys = meta::rename<KeyCtor, Keys>>
-            GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR auto convert_to(Tup && tup) {
-                return tuple_util::convert_to<HyMapKeys::template values>(wstd::forward<Tup>(tup));
+            GT_TARGET GT_FORCE_INLINE constexpr auto convert_to(Tup && tup) {
+                return tuple_util::convert_to<HyMapKeys::template values>(std::forward<Tup>(tup));
+            }
+
+            template <class Key, class Map>
+            GT_TARGET GT_FORCE_INLINE constexpr auto canonicalize_and_remove_key(Map && map) {
+                using res_t = from_meta_map<meta::mp_remove<hymap::to_meta_map<Map>, Key>>;
+                using generators_t = meta::transform<hymap_detail::at_generator_f, get_keys<res_t>>;
+                return tuple_util::generate<generators_t, res_t>(std::forward<Map>(map));
             }
 
             // This class holds two maps and models hymap content.
@@ -365,44 +459,44 @@ namespace gridtools {
 
                 friend struct merged_getter;
 
-                GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR base_t const &base() const { return *this; }
-                GT_TARGET GT_FORCE_INLINE base_t &base() { return *this; }
+                GT_TARGET GT_FORCE_INLINE constexpr base_t const &base() const { return *this; }
+                GT_TARGET GT_FORCE_INLINE constexpr base_t &base() { return *this; }
 
                 template <size_t I>
-                GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) source() const {
+                GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) source() const {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get < is_primary_index<I>::value ? 0 : 1 > (base());
                 }
                 template <size_t I>
-                GT_TARGET GT_FORCE_INLINE decltype(auto) source() {
+                GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) source() {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get < is_primary_index<I>::value ? 0 : 1 > (base());
                 }
 
                 template <size_t I>
-                GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) get() const {
+                GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) get() const {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<inner_index<I>::value>(source<I>());
                 }
                 template <size_t I>
-                GT_TARGET GT_FORCE_INLINE decltype(auto) get() {
+                GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) get() {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<inner_index<I>::value>(source<I>());
                 }
 
               public:
                 merged() = default;
 
-                GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR merged(Primary primary, Secondary secondary)
-                    : base_t(wstd::move(primary), wstd::move(secondary)) {}
+                GT_TARGET GT_FORCE_INLINE constexpr merged(Primary primary, Secondary secondary)
+                    : base_t(std::move(primary), std::move(secondary)) {}
 
-                GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR Primary const &primary() const {
+                GT_TARGET GT_FORCE_INLINE constexpr Primary const &primary() const {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<0>(base());
                 }
-                GT_TARGET GT_FORCE_INLINE Primary &primary() {
+                GT_TARGET GT_FORCE_INLINE constexpr Primary &primary() {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<0>(base());
                 }
 
-                GT_TARGET GT_FORCE_INLINE Secondary const &secondary() const {
+                GT_TARGET GT_FORCE_INLINE constexpr Secondary const &secondary() const {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<1>(base());
                 }
-                GT_TARGET GT_FORCE_INLINE Secondary &secondary() {
+                GT_TARGET GT_FORCE_INLINE constexpr Secondary &secondary() {
                     return tuple_util::GT_TARGET_NAMESPACE_NAME::get<1>(base());
                 }
 
@@ -414,12 +508,11 @@ namespace gridtools {
 
             struct merged_getter {
                 template <size_t I, class Primary, class Secondary>
-                static GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) get(
-                    merged<Primary, Secondary> const &obj) {
+                static GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) get(merged<Primary, Secondary> const &obj) {
                     return obj.template get<I>();
                 }
                 template <size_t I, class Primary, class Secondary>
-                static GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR decltype(auto) get(merged<Primary, Secondary> &obj) {
+                static GT_TARGET GT_FORCE_INLINE constexpr decltype(auto) get(merged<Primary, Secondary> &obj) {
                     return obj.template get<I>();
                 }
             };
@@ -431,17 +524,16 @@ namespace gridtools {
             // unlike concat keys could overlap
             // in the case of overlap the value is taken from the primary map
             template <class Primary, class Secondary>
-            GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR merged<Primary, Secondary> merge(
-                Primary primary, Secondary secondary) {
-                return {wstd::move(primary), wstd::move(secondary)};
+            GT_TARGET GT_FORCE_INLINE constexpr merged<Primary, Secondary> merge(Primary primary, Secondary secondary) {
+                return {std::move(primary), std::move(secondary)};
             }
 
             template <class Primary, class... Secondaries>
-            GT_TARGET GT_FORCE_INLINE GT_CONSTEXPR auto merge(Primary primary, Secondaries... secondaries) {
-                return merge(wstd::move(primary), merge(wstd::move(secondaries)...));
+            GT_TARGET GT_FORCE_INLINE constexpr auto merge(Primary primary, Secondaries... secondaries) {
+                return merge(std::move(primary), merge(std::move(secondaries)...));
             }
-        } // namespace hymap
-    }     // namespace hymap
+        }
+    } // namespace hymap
 } // namespace gridtools
 
 #endif // GT_TARGET_ITERATING
