@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -12,7 +12,9 @@
 #ifdef _OPENMP
 #include <omp.h>
 #else
+extern "C" {
 inline int omp_get_thread_num() { return 0; }
 inline int omp_get_max_threads() { return 1; }
 inline double omp_get_wtime() { return 0; }
+}
 #endif
