@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -12,7 +12,7 @@
 #include <vector>
 
 #include "../common/halo_descriptor.hpp"
-#include "../common/layout_map_metafunctions.hpp"
+#include "../common/layout_map.hpp"
 #include "high_level/descriptor_generic_manual.hpp"
 #include "high_level/descriptors.hpp"
 #include "high_level/descriptors_manual_gpu.hpp"
@@ -161,11 +161,10 @@ namespace gridtools {
             typename Gcl_Arch = cpu,
             int version = 0>
         class halo_exchange_dynamic_ut {
-
-            typedef typename reverse_map<T_layout_map>::type layout_map; // This is necessary since the internals of gcl
-                                                                         // use "increasing stride order" instead of
-                                                                         // "decreasing stride order"
-            typedef typename layout_transform<layout_map, layout2proc_map_abs>::type layout2proc_map;
+            // This is necessary since the internals of gcl use "increasing stride order" instead of "decreasing stride
+            // order"
+            using layout_map = reverse_map<T_layout_map>;
+            using layout2proc_map = layout_transform<layout_map, layout2proc_map_abs>;
 
           public:
             /**
@@ -248,7 +247,7 @@ namespace gridtools {
                \param[in] _fields data fields to be packed
             */
             template <typename... FIELDS>
-            void pack(const FIELDS *... _fields) {
+            void pack(const FIELDS *..._fields) {
                 hd.pack(_fields...);
             }
 
@@ -258,7 +257,7 @@ namespace gridtools {
                \param[in] _fields data fields where to unpack data
             */
             template <typename... FIELDS>
-            void unpack(FIELDS *... _fields) {
+            void unpack(FIELDS *..._fields) {
                 hd.unpack(_fields...);
             }
 
@@ -399,7 +398,7 @@ namespace gridtools {
                \param[in] _fields data fields to be packed
             */
             template <typename... FIELDS>
-            void pack(const FIELDS &... _fields) const {
+            void pack(const FIELDS &..._fields) const {
                 hd.pack(_fields...);
             }
 
@@ -409,7 +408,7 @@ namespace gridtools {
                \param[in] _fields data fields where to unpack data
             */
             template <typename... FIELDS>
-            void unpack(const FIELDS &... _fields) const {
+            void unpack(const FIELDS &..._fields) const {
                 hd.unpack(_fields...);
             }
 
