@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -12,19 +12,14 @@
 #include <vector>
 
 #include "../../common/array.hpp"
-#include "../../common/make_array.hpp"
-#include "../../common/numerics.hpp"
-
 #include "../low_level/Halo_Exchange_3D.hpp"
 #include "../low_level/proc_grids_3D.hpp"
-
-#include "../../common/boollist.hpp"
-#include "../../common/numerics.hpp"
 #include "../low_level/translate.hpp"
 #include "access.hpp"
 #include "descriptor_base.hpp"
 #include "empty_field_base.hpp"
 #include "helpers_impl.hpp"
+#include "numerics.hpp"
 
 namespace gridtools {
     namespace gcl {
@@ -112,7 +107,7 @@ namespace gridtools {
                may have different datatypes).
             */
             template <typename iterator, typename FIRST, typename... FIELDS>
-            void pack_all(array<int, DIMS> const &eta, iterator &it, FIRST const &field, const FIELDS &... args) const {
+            void pack_all(array<int, DIMS> const &eta, iterator &it, FIRST const &field, const FIELDS &...args) const {
                 pack(eta, field, it);
                 pack_all(eta, it, args...);
             }
@@ -136,7 +131,7 @@ namespace gridtools {
             */
             template <typename iterator, typename FIRST, typename... FIELDS>
             void unpack_all(
-                array<int, DIMS> const &eta, iterator &it, FIRST const &field, const FIELDS &... args) const {
+                array<int, DIMS> const &eta, iterator &it, FIRST const &field, const FIELDS &...args) const {
                 unpack(eta, field, it);
                 unpack_all(eta, it, args...);
             }
@@ -239,7 +234,7 @@ namespace gridtools {
             /**
                Type of the translation used to map dimensions to buffer addresses
              */
-            typedef translate_t<DIMS, typename default_layout_map<DIMS>::type> translate;
+            typedef translate_t<DIMS> translate;
 
           private:
             hndlr_descriptor_ut(hndlr_descriptor_ut const &) {}
@@ -399,7 +394,7 @@ namespace gridtools {
             /**
                Type of the translation used to map dimensions to buffer addresses
              */
-            typedef translate_t<DIMS, typename default_layout_map<DIMS>::type> translate;
+            typedef translate_t<DIMS> translate;
 
           private:
             hndlr_dynamic_ut(hndlr_dynamic_ut const &) = delete;
@@ -450,7 +445,7 @@ namespace gridtools {
                \param[in] _fields data fields to be packed
             */
             template <typename... FIELDS>
-            void pack(const FIELDS &... _fields) {
+            void pack(const FIELDS &..._fields) {
                 pack_dims<DIMS, 0>()(*this, _fields...);
             }
 
@@ -460,7 +455,7 @@ namespace gridtools {
                \param[in] _fields data fields where to unpack data
             */
             template <typename... FIELDS>
-            void unpack(const FIELDS &... _fields) const {
+            void unpack(const FIELDS &..._fields) const {
                 unpack_dims<DIMS, 0>()(*this, _fields...);
             }
 
@@ -500,19 +495,19 @@ namespace gridtools {
             template <int dummy>
             struct pack_dims<3, dummy> {
                 template <typename T, typename... FIELDS>
-                void operator()(T &hm, const FIELDS &... _fields) const {
+                void operator()(T &hm, const FIELDS &..._fields) const {
 #pragma omp parallel for schedule(dynamic, 1) collapse(3)
                     for (int ii = -1; ii <= 1; ++ii) {
                         for (int jj = -1; jj <= 1; ++jj) {
                             for (int kk = -1; kk <= 1; ++kk) {
                                 typedef proc_layout map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
                                 if ((ii != 0 || jj != 0 || kk != 0) &&
                                     (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                     DataType *it = &(hm.send_buffer[translate()(ii, jj, kk)][0]);
-                                    hm.halo.pack_all(make_array(ii, jj, kk), it, _fields...);
+                                    hm.halo.pack_all({ii, jj, kk}, it, _fields...);
 
                                     hm.m_haloexch.set_send_to_size(
                                         hm.send_size[translate()(ii, jj, kk)] * sizeof...(_fields) * sizeof(DataType),
@@ -537,19 +532,19 @@ namespace gridtools {
             template <int dummy>
             struct unpack_dims<3, dummy> {
                 template <typename T, typename... FIELDS>
-                void operator()(const T &hm, const FIELDS &... _fields) const {
+                void operator()(const T &hm, const FIELDS &..._fields) const {
 #pragma omp parallel for schedule(dynamic, 1) collapse(3)
                     for (int ii = -1; ii <= 1; ++ii) {
                         for (int jj = -1; jj <= 1; ++jj) {
                             for (int kk = -1; kk <= 1; ++kk) {
                                 typedef proc_layout map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
                                 if ((ii != 0 || jj != 0 || kk != 0) &&
                                     (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                     DataType *it = &(hm.recv_buffer[translate()(ii, jj, kk)][0]);
-                                    hm.halo.unpack_all(make_array(ii, jj, kk), it, _fields...);
+                                    hm.halo.unpack_all({ii, jj, kk}, it, _fields...);
                                 }
                             }
                         }
@@ -569,14 +564,14 @@ namespace gridtools {
                         for (int jj = -1; jj <= 1; ++jj) {
                             for (int kk = -1; kk <= 1; ++kk) {
                                 typedef proc_layout map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
                                 if ((ii != 0 || jj != 0 || kk != 0) &&
                                     (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                     DataType *it = &(hm.send_buffer[translate()(ii, jj, kk)][0]);
                                     for (size_t i = 0; i < fields.size(); ++i) {
-                                        hm.halo.pack(make_array(ii, jj, kk), fields[i], it);
+                                        hm.halo.pack({ii, jj, kk}, fields[i], it);
                                     }
 
                                     hm.m_haloexch.set_send_to_size(
@@ -608,14 +603,14 @@ namespace gridtools {
                         for (int jj = -1; jj <= 1; ++jj) {
                             for (int kk = -1; kk <= 1; ++kk) {
                                 typedef proc_layout map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
                                 if ((ii != 0 || jj != 0 || kk != 0) &&
                                     (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                     DataType *it = &(hm.recv_buffer[translate()(ii, jj, kk)][0]);
                                     for (size_t i = 0; i < fields.size(); ++i) {
-                                        hm.halo.unpack(make_array(ii, jj, kk), fields[i], it);
+                                        hm.halo.unpack({ii, jj, kk}, fields[i], it);
                                     }
                                 }
                             }
