@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -11,10 +11,9 @@
 #include <utility>
 
 #include "../../common/defs.hpp"
-#include "../../common/numerics.hpp"
 #include "empty_field_base.hpp"
-
 #include "helpers_impl.hpp"
+#include "numerics.hpp"
 
 #ifdef GT_CUDACC
 #include "m_packXL.hpp"
@@ -118,7 +117,7 @@ namespace gridtools {
             /**
                Type of the translation used to map dimensions to buffer addresses
              */
-            typedef translate_t<DIMS, typename default_layout_map<DIMS>::type> translate;
+            typedef translate_t<DIMS> translate;
 
           private:
             hndlr_dynamic_ut(hndlr_dynamic_ut const &) = delete;
@@ -164,7 +163,7 @@ namespace gridtools {
             */
             void setup(const int max_fields_n) {
 
-                typedef translate_t<3, default_layout_map<3>::type> translate;
+                typedef translate_t<3> translate;
                 typedef translate_t<3, proc_layout> translate_P;
 
                 dangeroushalo[0] = halo.dangerous_raw_array()[0];
@@ -179,9 +178,9 @@ namespace gridtools {
                     int ii = 1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[0].reset_minus();
                         dangeroushalo_r[0].reset_plus();
@@ -192,9 +191,9 @@ namespace gridtools {
                     int ii = -1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[0].reset_plus();
                         dangeroushalo_r[0].reset_minus();
@@ -205,9 +204,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[1].reset_minus();
                         dangeroushalo_r[1].reset_plus();
@@ -218,9 +217,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = -1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[1].reset_plus();
                         dangeroushalo_r[1].reset_minus();
@@ -231,9 +230,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = 1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[2].reset_minus();
                         dangeroushalo_r[2].reset_plus();
@@ -244,9 +243,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = -1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         dangeroushalo[2].reset_plus();
                         dangeroushalo_r[2].reset_minus();
@@ -258,12 +257,12 @@ namespace gridtools {
                         for (int kk = -1; kk <= 1; ++kk)
                             if (ii != 0 || jj != 0 || kk != 0) {
                                 typedef typename translate_P::map_type map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
 
                                 if (base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1) {
-                                    send_size[translate()(ii, jj, kk)] = halo.send_buffer_size(make_array(ii, jj, kk));
+                                    send_size[translate()(ii, jj, kk)] = halo.send_buffer_size({ii, jj, kk});
 
                                     send_buffer[translate()(ii, jj, kk)] = gcl_alloc<DataType, arch_type>::alloc(
                                         send_size[translate()(ii, jj, kk)] * max_fields_n);
@@ -275,7 +274,7 @@ namespace gridtools {
                                         jj_P,
                                         kk_P);
 
-                                    recv_size[translate()(ii, jj, kk)] = halo.recv_buffer_size(make_array(ii, jj, kk));
+                                    recv_size[translate()(ii, jj, kk)] = halo.recv_buffer_size({ii, jj, kk});
 
                                     recv_buffer[translate()(ii, jj, kk)] = gcl_alloc<DataType, arch_type>::alloc(
                                         recv_size[translate()(ii, jj, kk)] * max_fields_n);
@@ -341,8 +340,8 @@ namespace gridtools {
                \param[in] fields vector with data fields pointers to be packed from
             */
             template <typename... Pointers>
-            void pack(const Pointers *... fields) {
-                typedef translate_t<3, default_layout_map<3>::type> translate;
+            void pack(const Pointers *...fields) {
+                typedef translate_t<3> translate;
                 auto ints = std::make_integer_sequence<unsigned int, sizeof...(Pointers)>{};
                 if (send_size[translate()(0, 0, -1)]) {
                     m_packZL_variadic(
@@ -380,9 +379,9 @@ namespace gridtools {
                             if (ii != 0 || jj != 0 || kk != 0) {
                                 using translate_P = translate_t<3, proc_layout>;
                                 using map_type = typename translate_P::map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
 
                                 if (base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1) {
                                     base_type::m_haloexch.set_send_to_size(
@@ -402,9 +401,9 @@ namespace gridtools {
             }
 
             template <typename... Pointers>
-            void unpack(Pointers *... fields) {
+            void unpack(Pointers *...fields) {
                 auto ints = std::make_integer_sequence<unsigned int, sizeof...(Pointers)>{};
-                typedef translate_t<3, default_layout_map<3>::type> translate;
+                typedef translate_t<3> translate;
                 if (recv_size[translate()(0, 0, -1)]) {
                     m_unpackZL_variadic(
                         d_recv_buffer, d_recv_size, dangeroushalo_r, halo_d_r, std::make_tuple(fields...), ints);
@@ -437,7 +436,7 @@ namespace gridtools {
                \param[in] fields vector with data fields pointers to be packed from
             */
             void pack(std::vector<DataType *> const &fields) {
-                typedef translate_t<3, default_layout_map<3>::type> translate;
+                typedef translate_t<3> translate;
                 if (send_size[translate()(0, 0, -1)]) {
                     m_packZL(fields, d_send_buffer, d_send_size, dangeroushalo, halo_d);
                 }
@@ -463,9 +462,9 @@ namespace gridtools {
                             if (ii != 0 || jj != 0 || kk != 0) {
                                 using translate_P = translate_t<3, proc_layout>;
                                 using map_type = typename translate_P::map_type;
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
 
                                 if (base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1) {
                                     base_type::m_haloexch.set_send_to_size(
@@ -493,7 +492,7 @@ namespace gridtools {
                \param[in] fields vector with data fields pointers to be unpacked into
             */
             void unpack(std::vector<DataType *> const &fields) {
-                typedef translate_t<3, default_layout_map<3>::type> translate;
+                typedef translate_t<3> translate;
                 if (recv_size[translate()(0, 0, -1)]) {
                     m_unpackZL(fields, d_recv_buffer, d_recv_size, dangeroushalo_r, halo_d_r);
                 }
