@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -12,8 +12,6 @@
 
 #include <type_traits>
 
-#include "type_traits.hpp"
-
 namespace gridtools {
     namespace meta {
         /**
@@ -22,6 +20,6 @@ namespace gridtools {
         template <class, class = void>
         struct has_type : std::false_type {};
         template <class T>
-        struct has_type<T, void_t<typename T::type>> : std::true_type {};
+        struct has_type<T, std::void_t<typename T::type>> : std::true_type {};
     } // namespace meta
 } // namespace gridtools
