@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -26,11 +26,11 @@ namespace gridtools {
 
             template <class State, template <class...> class L, class Key, class... Vals>
             struct mp_inverse_helper<State, L<Key, Vals...>>
-                : lfold<meta::mp_insert, State, meta::list<L<Vals, Key>...>> {};
+                : foldl<meta::mp_insert, State, meta::list<L<Vals, Key>...>> {};
         } // namespace lazy
         GT_META_DELEGATE_TO_LAZY(mp_inverse_helper, (class State, class Item), (State, Item));
 
         template <class Src>
-        using mp_inverse = lfold<mp_inverse_helper, clear<Src>, Src>;
+        using mp_inverse = foldl<mp_inverse_helper, clear<Src>, Src>;
     } // namespace meta
 } // namespace gridtools
