@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -19,7 +19,6 @@
 #include "../common/host_device.hpp"
 #include "../common/integral_constant.hpp"
 #include "../common/tuple.hpp"
-#include "../meta/type_traits.hpp"
 #include "concept.hpp"
 
 namespace gridtools {
@@ -28,7 +27,7 @@ namespace gridtools {
 
             template <class Key, class T>
             struct generic_loop {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 T m_num_steps;
                 T m_step;
@@ -40,7 +39,7 @@ namespace gridtools {
                     T m_step;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, Strides const &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, Strides const &strides) const {
                         assert(m_num_steps >= 0);
                         if (m_num_steps <= 0)
                             return;
@@ -49,13 +48,13 @@ namespace gridtools {
                             m_fun(ptr, strides);
                             shift(ptr, stride, m_step);
                         }
-                        shift(ptr, stride, -m_step * m_num_steps);
+                        shift(std::forward<Ptr>(ptr), stride, -m_step * m_num_steps);
                     }
                 };
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun), m_num_steps, m_step};
+                constexpr GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
+                    return {std::forward<Fun>(fun), m_num_steps, m_step};
                 }
 
                 template <class Outer>
@@ -66,16 +65,16 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         assert(m_num_steps >= 0);
                         if (m_num_steps <= 0)
                             return;
                         if (++m_pos == m_num_steps) {
                             shift(ptr, get_stride<Key>(strides), m_step * (1 - m_num_steps));
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         } else {
-                            shift(ptr, get_stride<Key>(strides), m_step);
+                            shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), m_step);
                         }
                     }
 
@@ -83,8 +82,8 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), m_num_steps, m_step, 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), m_num_steps, m_step, 0};
                 }
 
                 struct outer_most_cursor_f {
@@ -92,20 +91,20 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         --m_pos;
-                        shift(ptr, get_stride<Key>(strides), m_step);
+                        shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), m_step);
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_step, m_num_steps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_step, m_num_steps}; }
             };
 
             template <class Key, class T, ptrdiff_t Step>
             struct known_step_loop {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 T m_num_steps;
 
@@ -115,7 +114,7 @@ namespace gridtools {
                     T m_num_steps;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, Strides const &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, Strides const &strides) const {
                         assert(m_num_steps >= 0);
                         if (m_num_steps <= 0)
                             return;
@@ -125,13 +124,13 @@ namespace gridtools {
                             shift(ptr, stride, integral_constant<T, Step>{});
                         }
                         static constexpr T minus_step = -Step;
-                        shift(ptr, stride, minus_step * m_num_steps);
+                        shift(std::forward<Ptr>(ptr), stride, minus_step * m_num_steps);
                     }
                 };
 
                 template <class Fun>
                 GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun), m_num_steps};
+                    return {std::forward<Fun>(fun), m_num_steps};
                 }
 
                 template <class Outer>
@@ -141,16 +140,16 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         assert(m_num_steps >= 0);
                         if (m_num_steps <= 0)
                             return;
                         if (++m_pos == m_num_steps) {
                             shift(ptr, get_stride<Key>(strides), Step * (1 - m_num_steps));
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         } else {
-                            shift(ptr, get_stride<Key>(strides), integral_constant<T, Step>{});
+                            shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), integral_constant<T, Step>{});
                         }
                     }
 
@@ -158,28 +157,28 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), m_num_steps, 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), m_num_steps, 0};
                 }
 
                 struct outer_most_cursor_f {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         --m_pos;
-                        shift(ptr, get_stride<Key>(strides), integral_constant<T, Step>{});
+                        shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), integral_constant<T, Step>{});
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_num_steps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_num_steps}; }
             };
 
             template <class Key, class T>
             struct known_step_loop<Key, T, 0> {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 T m_num_steps;
 
@@ -189,16 +188,16 @@ namespace gridtools {
                     T m_num_steps;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, const Strides &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, const Strides &strides) const {
                         assert(m_num_steps >= 0);
                         for (T i = 0; i < m_num_steps; ++i)
-                            m_fun(ptr, strides);
+                            m_fun(std::forward<Ptr>(ptr), strides);
                     }
                 };
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun), m_num_steps};
+                constexpr GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
+                    return {std::forward<Fun>(fun), m_num_steps};
                 }
 
                 template <class Outer>
@@ -208,13 +207,13 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         assert(m_num_steps >= 0);
                         if (m_num_steps <= 0)
                             return;
                         if (++m_pos == m_num_steps) {
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         }
                     }
 
@@ -222,27 +221,27 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), m_num_steps, 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), m_num_steps, 0};
                 }
 
                 struct outer_most_cursor_f {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&, Strides const &) {
                         --m_pos;
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_num_steps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_num_steps}; }
             };
 
             template <class Key, class T, T NumSteps>
             struct known_num_steps_loop {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 T m_step;
 
@@ -252,7 +251,7 @@ namespace gridtools {
                     T m_step;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, const Strides &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, const Strides &strides) const {
                         auto &&stride = get_stride<Key>(strides);
                         // TODO(anstaf): to figure out if for_each<make_indices_c<NumSteps>>(...) produces better code.
                         for (T i = 0; i < NumSteps; ++i) {
@@ -260,13 +259,13 @@ namespace gridtools {
                             shift(ptr, stride, m_step);
                         }
                         static constexpr T minus_num_steps = -NumSteps;
-                        shift(ptr, stride, m_step * minus_num_steps);
+                        shift(std::forward<Ptr>(ptr), stride, m_step * minus_num_steps);
                     }
                 };
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun), m_step};
+                constexpr GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
+                    return {std::forward<Fun>(fun), m_step};
                 }
 
                 template <class Outer>
@@ -276,14 +275,14 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         if (++m_pos == NumSteps) {
                             constexpr T num_steps_back = 1 - NumSteps;
                             shift(ptr, get_stride<Key>(strides), m_step * num_steps_back);
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         } else {
-                            shift(ptr, get_stride<Key>(strides), m_step);
+                            shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), m_step);
                         }
                     }
 
@@ -291,8 +290,8 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), m_step, 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), m_step, 0};
                 }
 
                 struct outer_most_cursor_f {
@@ -300,39 +299,39 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         --m_pos;
-                        shift(ptr, get_stride<Key>(strides), m_step);
+                        shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), m_step);
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_step, NumSteps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {m_step, NumSteps}; }
             };
 
             template <class Key, class T, ptrdiff_t NumSteps, ptrdiff_t Step>
             struct all_known_loop {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 template <class Fun>
                 struct loop_f {
                     Fun m_fun;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, const Strides &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, const Strides &strides) const {
                         auto &&stride = get_stride<Key>(strides);
                         for (T i = 0; i < (T)NumSteps; ++i) {
                             m_fun(ptr, strides);
                             shift(ptr, stride, integral_constant<T, Step>{});
                         }
-                        shift(ptr, stride, integral_constant<T, -Step * NumSteps>{});
+                        shift(std::forward<Ptr>(ptr), stride, integral_constant<T, -Step * NumSteps>{});
                     }
                 };
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun)};
+                constexpr GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
+                    return {std::forward<Fun>(fun)};
                 }
 
                 template <class Outer>
@@ -341,14 +340,14 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         if (++m_pos == NumSteps) {
                             constexpr T offset_back = Step * (1 - NumSteps);
                             shift(ptr, get_stride<Key>(strides), offset_back);
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         } else {
-                            shift(ptr, get_stride<Key>(strides), Step);
+                            shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), Step);
                         }
                     }
 
@@ -356,43 +355,43 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), 0};
                 }
 
                 struct outer_most_cursor_f {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         --m_pos;
-                        shift(ptr, get_stride<Key>(strides), Step);
+                        shift(std::forward<Ptr>(ptr), get_stride<Key>(strides), Step);
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {NumSteps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {NumSteps}; }
             };
 
             template <class Key, class T, ptrdiff_t NumSteps>
             struct all_known_loop<Key, T, NumSteps, 0> {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 template <class Fun>
                 struct loop_f {
                     Fun m_fun;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION operator()(Ptr &ptr, Strides const &strides) const {
+                    void GT_FUNCTION operator()(Ptr &&ptr, Strides const &strides) const {
                         for (T i = 0; i < (T)NumSteps; ++i)
                             m_fun(ptr, strides);
                     }
                 };
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
-                    return {wstd::forward<Fun>(fun)};
+                constexpr GT_FUNCTION loop_f<Fun> operator()(Fun &&fun) const {
+                    return {std::forward<Fun>(fun)};
                 }
 
                 template <class Outer>
@@ -401,10 +400,10 @@ namespace gridtools {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &ptr, Strides const &strides) {
+                    void GT_FUNCTION next(Ptr &&ptr, Strides const &strides) {
                         if (++m_pos == NumSteps) {
                             m_pos = 0;
-                            m_outer.next(ptr, strides);
+                            m_outer.next(std::forward<Ptr>(ptr), strides);
                         }
                     }
 
@@ -412,91 +411,91 @@ namespace gridtools {
                 };
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
-                    return {wstd::forward<Outer>(outer), 0};
+                constexpr GT_FUNCTION cursor_f<Outer> make_cursor(Outer &&outer) const {
+                    return {std::forward<Outer>(outer), 0};
                 }
 
                 struct outer_most_cursor_f {
                     T m_pos;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &, Strides const &) {
+                    void GT_FUNCTION next(Ptr &&, Strides const &) {
                         --m_pos;
                     }
 
                     GT_FUNCTION bool done() const { return m_pos > 0; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {NumSteps}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {NumSteps}; }
             };
 
             template <class Key, class T>
             struct all_known_loop<Key, T, 1, 0> {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION Fun operator()(Fun &&fun) const {
+                constexpr GT_FUNCTION Fun operator()(Fun &&fun) const {
                     return fun;
                 }
 
                 template <class Outer>
-                GT_CONSTEXPR GT_FUNCTION Outer make_cursor(Outer &&outer) const {
-                    return wstd::forward<Outer>(outer);
+                constexpr GT_FUNCTION Outer make_cursor(Outer &&outer) const {
+                    return std::forward<Outer>(outer);
                 }
 
                 struct outer_most_cursor_f {
                     bool m_done;
 
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &, Strides const &) {
+                    void GT_FUNCTION next(Ptr &&, Strides const &) {
                         m_done = true;
                     }
 
                     GT_FUNCTION bool done() const { return m_done; }
                 };
 
-                GT_CONSTEXPR GT_FUNCTION outer_most_cursor_f make_cursor() const { return {false}; }
+                constexpr GT_FUNCTION outer_most_cursor_f make_cursor() const { return {false}; }
             };
 
             template <class Key, class T>
             struct all_known_loop<Key, T, 0, 0> {
-                static_assert(std::is_signed<T>::value, GT_INTERNAL_ERROR);
+                static_assert(std::is_signed_v<T>, GT_INTERNAL_ERROR);
 
                 template <class Fun>
-                GT_CONSTEXPR GT_FUNCTION gridtools::host_device::noop operator()(Fun &&) const {
+                constexpr GT_FUNCTION gridtools::host_device::noop operator()(Fun &&) const {
                     return {};
                 }
 
                 struct cursor_f {
                     template <class Ptr, class Strides>
-                    void GT_FUNCTION next(Ptr &, Strides const &) {}
+                    void GT_FUNCTION next(Ptr &&, Strides const &) {}
 
                     GT_FUNCTION bool done() const { return true; }
                 };
 
                 template <class... Ts>
-                GT_CONSTEXPR GT_FUNCTION cursor_f make_cursor(Ts &&...) const {
+                constexpr GT_FUNCTION cursor_f make_cursor(Ts &&...) const {
                     return {};
                 }
             };
 
             struct make_cursor_f {
                 template <class Cursor, class Loop>
-                GT_CONSTEXPR GT_FUNCTION auto operator()(Cursor &&cursor, Loop const &loop) const {
-                    return loop.make_cursor(wstd::forward<Cursor>(cursor));
+                constexpr GT_FUNCTION auto operator()(Cursor &&cursor, Loop const &loop) const {
+                    return loop.make_cursor(std::forward<Cursor>(cursor));
                 }
             };
 
             template <class Loops>
-            GT_CONSTEXPR GT_FUNCTION auto make_cursor_r(Loops &&loops) {
+            constexpr GT_FUNCTION auto make_cursor_r(Loops &&loops) {
                 return tuple_util::host_device::fold(make_cursor_f{},
-                    tuple_util::host_device::get<0>(wstd::forward<Loops>(loops)).make_cursor(),
-                    tuple_util::host_device::drop_front<1>(wstd::forward<Loops>(loops)));
+                    tuple_util::host_device::get<0>(std::forward<Loops>(loops)).make_cursor(),
+                    tuple_util::host_device::drop_front<1>(std::forward<Loops>(loops)));
             }
 
             template <class Loops>
-            GT_CONSTEXPR GT_FUNCTION auto make_cursor(Loops &&loops) {
-                return make_cursor_r(tuple_util::host_device::reverse(wstd::forward<Loops>(loops)));
+            constexpr GT_FUNCTION auto make_cursor(Loops &&loops) {
+                return make_cursor_r(tuple_util::host_device::reverse(std::forward<Loops>(loops)));
             }
 
             template <class Ptr, class Strides, class Cursor>
@@ -517,9 +516,9 @@ namespace gridtools {
             };
 
             template <class Ptr, class Strides, class Cursor>
-            GT_CONSTEXPR GT_FUNCTION range<Ptr, Strides const &, Cursor> make_range(
+            constexpr GT_FUNCTION range<Ptr, Strides const &, Cursor> make_range(
                 Ptr ptr, Strides const &strides, Cursor &&cursor) {
-                return {wstd::move(ptr), strides, wstd::forward<Cursor>(cursor)};
+                return {std::move(ptr), strides, std::forward<Cursor>(cursor)};
             }
         } // namespace loop_impl_
 
@@ -530,7 +529,7 @@ namespace gridtools {
          *   @param num_steps number of iterations in the loop. Can be of integral or integral_constant type
          *   @param step (optional) a step for each iteration. Can be of integral or integral_constant type.
          *               The default is integral_constant<int, 1>
-         *   @return a functor that accepts another functor with the signature: `void(Ptr&, Strides const&)` and
+         *   @return a functor that accepts another functor with the signature: `void(Ptr&&, Strides const&)` and
          *           returns a functor also with the same signature.
          *
          *   Usage:
@@ -587,9 +586,9 @@ namespace gridtools {
             class T1,
             class T2,
             class T = std::common_type_t<T1, T2>,
-            std::enable_if_t<std::is_integral<T1>::value && std::is_integral<T2>::value, int> = 0>
-        GT_CONSTEXPR GT_FUNCTION loop_impl_::generic_loop<Key, std::make_signed_t<T>> make_loop(T1 num_steps, T2 step) {
-            return {num_steps, step};
+            std::enable_if_t<std::is_integral_v<T1> && std::is_integral_v<T2>, int> = 0>
+        constexpr GT_FUNCTION loop_impl_::generic_loop<Key, std::make_signed_t<T>> make_loop(T1 num_steps, T2 step) {
+            return {std::make_signed_t<T>(num_steps), step};
         }
 
         template <class Key,
@@ -597,10 +596,10 @@ namespace gridtools {
             class T2 = int,
             T2 Step = 1,
             class T = std::common_type_t<T1, T2>,
-            std::enable_if_t<std::is_integral<T1>::value, int> = 0>
-        GT_CONSTEXPR GT_FUNCTION loop_impl_::known_step_loop<Key, std::make_signed_t<T>, Step> make_loop(
+            std::enable_if_t<std::is_integral_v<T1>, int> = 0>
+        constexpr GT_FUNCTION loop_impl_::known_step_loop<Key, std::make_signed_t<T>, Step> make_loop(
             T1 num_steps, std::integral_constant<T2, Step> = {}) {
-            return {num_steps};
+            return {std::make_signed_t<T>(num_steps)};
         }
 
         template <class Key,
@@ -608,8 +607,8 @@ namespace gridtools {
             T1 NumStepsV,
             class T2,
             class T = std::common_type_t<T1, T2>,
-            std::enable_if_t<std::is_integral<T1>::value && (NumStepsV > 1), int> = 0>
-        GT_CONSTEXPR GT_FUNCTION loop_impl_::known_num_steps_loop<Key, std::make_signed_t<T>, NumStepsV> make_loop(
+            std::enable_if_t<std::is_integral_v<T1> && (NumStepsV > 1), int> = 0>
+        constexpr GT_FUNCTION loop_impl_::known_num_steps_loop<Key, std::make_signed_t<T>, NumStepsV> make_loop(
             std::integral_constant<T1, NumStepsV>, T2 step) {
             return {step};
         }
@@ -619,8 +618,8 @@ namespace gridtools {
             T1 NumStepsV,
             class T2,
             class T = std::common_type_t<T1, T2>,
-            std::enable_if_t<std::is_integral<T1>::value && (NumStepsV == 0 || NumStepsV == 1), int> = 0>
-        GT_CONSTEXPR GT_FUNCTION loop_impl_::all_known_loop<Key, std::make_signed_t<T>, NumStepsV, 0> make_loop(
+            std::enable_if_t<std::is_integral_v<T1> && (NumStepsV == 0 || NumStepsV == 1), int> = 0>
+        constexpr GT_FUNCTION loop_impl_::all_known_loop<Key, std::make_signed_t<T>, NumStepsV, 0> make_loop(
             std::integral_constant<T1, NumStepsV>, T2) {
             return {};
         }
@@ -632,8 +631,8 @@ namespace gridtools {
             T2 StepV = 1,
             class T = std::common_type_t<T1, T2>,
             std::enable_if_t<(NumStepsV >= 0), int> = 0>
-        GT_CONSTEXPR
-            GT_FUNCTION loop_impl_::all_known_loop<Key, std::make_signed_t<T>, NumStepsV, (NumStepsV > 1) ? StepV : 0>
+        constexpr GT_FUNCTION
+            loop_impl_::all_known_loop<Key, std::make_signed_t<T>, NumStepsV, (NumStepsV > 1) ? StepV : 0>
             make_loop(std::integral_constant<T1, NumStepsV>, std::integral_constant<T2, StepV> = {}) {
             return {};
         }
@@ -653,12 +652,12 @@ namespace gridtools {
          *   }
          */
         template <class Ptr, class Strides, class OuterMostLoop, class... Loops>
-        GT_CONSTEXPR GT_FUNCTION auto make_range(
-            Ptr ptr, Strides const &strides, OuterMostLoop &&outer_most_loop, Loops &&... loops) {
-            return loop_impl_::make_range(wstd::move(ptr),
+        constexpr GT_FUNCTION auto make_range(
+            Ptr ptr, Strides const &strides, OuterMostLoop &&outer_most_loop, Loops &&...loops) {
+            return loop_impl_::make_range(std::move(ptr),
                 strides,
                 loop_impl_::make_cursor(tuple<OuterMostLoop, Loops...>{
-                    wstd::forward<OuterMostLoop>(outer_most_loop), wstd::forward<Loops>(loops)...}));
+                    std::forward<OuterMostLoop>(outer_most_loop), std::forward<Loops>(loops)...}));
         }
 
     } // namespace sid
