@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -20,7 +20,7 @@ namespace gridtools {
             struct is_tmp_arg : std::false_type {};
 
             template <class T>
-            struct is_tmp_arg<T, void_t<typename T::tmp_tag>> : std::true_type {};
+            struct is_tmp_arg<T, std::void_t<typename T::tmp_tag>> : std::true_type {};
         } // namespace core
     }     // namespace stencil
 } // namespace gridtools
