@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -29,8 +29,8 @@ namespace gridtools {
                 GT_FUNCTION constexpr dimension(int_t value = 0) : value(value) {}
                 int_t value;
 
-                friend GT_FUNCTION GT_CONSTEXPR dimension operator+(dimension, int_t offset) { return {offset}; }
-                friend GT_FUNCTION GT_CONSTEXPR dimension operator-(dimension, int_t offset) { return {-offset}; }
+                friend GT_FUNCTION constexpr dimension operator+(dimension, int_t offset) { return {offset}; }
+                friend GT_FUNCTION constexpr dimension operator-(dimension, int_t offset) { return {-offset}; }
             };
         } // namespace cartesian
     }     // namespace stencil
