@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -19,13 +19,13 @@ namespace gridtools {
             namespace expressions {
                 struct divide_f {
                     template <class Lhs, class Rhs>
-                    GT_FUNCTION GT_CONSTEXPR auto operator()(Lhs const &lhs, Rhs const &rhs) const {
+                    GT_FUNCTION constexpr auto operator()(Lhs const &lhs, Rhs const &rhs) const {
                         return lhs / rhs;
                     }
                 };
 
                 template <class Lhs, class Rhs>
-                GT_FUNCTION GT_CONSTEXPR auto operator/(Lhs lhs, Rhs rhs)
+                GT_FUNCTION constexpr auto operator/(Lhs lhs, Rhs rhs)
                     -> decltype(make_expr(divide_f(), Lhs(), Rhs())) {
                     return make_expr(divide_f(), lhs, rhs);
                 }
