@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -19,23 +19,22 @@ namespace gridtools {
             namespace expressions {
                 struct plus_f {
                     template <class Lhs, class Rhs>
-                    GT_FUNCTION GT_CONSTEXPR auto operator()(Lhs const &lhs, Rhs const &rhs) const {
+                    GT_FUNCTION constexpr auto operator()(Lhs const &lhs, Rhs const &rhs) const {
                         return lhs + rhs;
                     }
                     template <class Arg>
-                    GT_FUNCTION GT_CONSTEXPR auto operator()(Arg const &arg) const {
+                    GT_FUNCTION constexpr auto operator()(Arg const &arg) const {
                         return +arg;
                     }
                 };
 
                 template <class Lhs, class Rhs>
-                GT_FUNCTION GT_CONSTEXPR auto operator+(Lhs lhs, Rhs rhs)
-                    -> decltype(make_expr(plus_f(), Lhs(), Rhs())) {
+                GT_FUNCTION constexpr auto operator+(Lhs lhs, Rhs rhs) -> decltype(make_expr(plus_f(), Lhs(), Rhs())) {
                     return make_expr(plus_f(), lhs, rhs);
                 }
 
                 template <class Arg>
-                GT_FUNCTION GT_CONSTEXPR auto operator+(Arg arg) -> decltype(make_expr(plus_f(), Arg())) {
+                GT_FUNCTION constexpr auto operator+(Arg arg) -> decltype(make_expr(plus_f(), Arg())) {
                     return make_expr(plus_f(), arg);
                 }
             } // namespace expressions
