@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -20,13 +20,13 @@ namespace gridtools {
                 template <int I>
                 struct pow_f {
                     template <class Arg>
-                    GT_FUNCTION GT_CONSTEXPR auto operator()(Arg const &arg) const {
+                    GT_FUNCTION constexpr auto operator()(Arg const &arg) const {
                         return gt_pow<I>::template apply(arg);
                     }
                 };
 
                 template <int I, class Arg>
-                GT_FUNCTION GT_CONSTEXPR auto pow(Arg arg) -> decltype(make_expr(pow_f<I>(), Arg())) {
+                GT_FUNCTION constexpr auto pow(Arg arg) -> decltype(make_expr(pow_f<I>(), Arg())) {
                     return make_expr(pow_f<I>(), arg);
                 }
             } // namespace expressions
