@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -10,10 +10,12 @@
 
 #pragma once
 
+#include <cassert>
 #include <type_traits>
 #include <utility>
 
 #include "../../common/defs.hpp"
+#include "../../common/for_each.hpp"
 #include "../../common/host_device.hpp"
 #include "../../common/hymap.hpp"
 #include "../../common/integral_constant.hpp"
@@ -90,8 +92,8 @@ namespace gridtools {
                     template <class PlhInfo,
                         class Cached,
                         class Orig,
-                        std::enable_if_t<std::is_same<typename PlhInfo::cache_io_policies_t,
-                                             meta::list<cache_io_policy::fill>>::value,
+                        std::enable_if_t<
+                            std::is_same_v<typename PlhInfo::cache_io_policies_t, meta::list<cache_io_policy::fill>>,
                             int> = 0>
                     GT_FUNCTION void sync(Cached cached, Orig orig) {
                         *cached = *orig;
@@ -100,8 +102,8 @@ namespace gridtools {
                     template <class PlhInfo,
                         class Cached,
                         class Orig,
-                        std::enable_if_t<std::is_same<typename PlhInfo::cache_io_policies_t,
-                                             meta::list<cache_io_policy::flush>>::value,
+                        std::enable_if_t<
+                            std::is_same_v<typename PlhInfo::cache_io_policies_t, meta::list<cache_io_policy::flush>>,
                             int> = 0>
                     GT_FUNCTION void sync(Cached cached, Orig orig) {
                         *orig = *cached;
@@ -217,7 +219,7 @@ namespace gridtools {
                     template <uint_t Splitter, int_t OffsetLimit, int_t FromOffset, int_t ToOffset, int_t Lim>
                     struct levels_are_close<be_api::level<Splitter, FromOffset, OffsetLimit>,
                         be_api::level<Splitter, ToOffset, OffsetLimit>,
-                        Lim> : bool_constant<(real_offset(ToOffset) - real_offset(FromOffset) < Lim)> {};
+                        Lim> : std::bool_constant<(real_offset(ToOffset) - real_offset(FromOffset) < Lim)> {};
 
                     template <class PlhInfo,
                         class Execution,
@@ -225,28 +227,45 @@ namespace gridtools {
                         class LastInterval,
                         class CurInterval>
                     struct make_sync_fun {
-                        static constexpr bool is_fill = std::is_same<typename PlhInfo::cache_io_policies_t,
-                            meta::list<cache_io_policy::fill>>::value;
-                        static constexpr bool is_first = std::is_same<FirstInterval, CurInterval>::value;
-                        static constexpr bool is_last = std::is_same<LastInterval, CurInterval>::value;
+                        static constexpr bool is_fill =
+                            std::is_same_v<typename PlhInfo::cache_io_policies_t, meta::list<cache_io_policy::fill>>;
+                        static constexpr bool is_first = std::is_same_v<FirstInterval, CurInterval>;
+                        static constexpr bool is_last = std::is_same_v<LastInterval, CurInterval>;
                         static constexpr int_t minus = PlhInfo::extent_t::kminus::value;
                         static constexpr int_t plus = PlhInfo::extent_t::kplus::value;
                         static constexpr bool close_to_first =
                             levels_are_close<meta::first<FirstInterval>, meta::second<CurInterval>, -minus>::value;
                         static constexpr bool close_to_last =
                             levels_are_close<meta::first<CurInterval>, meta::second<LastInterval>, plus>::value;
+
+                        //  Those static asserts are commented on purpose.
+                        //  They trigger when the filling or the flushing of the k-cache could cause access violation in
+                        //   the "inner" (runtime size) intervals due to the small offset limit.
+                        //  We optimistically assume that the user knows what he is doing in this case.
+                        //
+                        //  static_assert(
+                        //      levels_are_close<meta::first<FirstInterval>, meta::first<CurInterval>, -minus>::value ==
+                        //      close_to_first, "offset_limit too small");
+                        //  static_assert(
+                        //      levels_are_close<meta::second<CurInterval>, meta::second<LastInterval>, plus>::value ==
+                        //      close_to_last, "offset_limit too small");
+
                         static constexpr bool is_forward = !be_api::is_backward<Execution>::value;
 
                         static constexpr bool sync_all = is_forward == is_fill ? is_first : is_last;
 
-                        static constexpr range range_v =
-                            minus == plus ? range::minus
-                                          : sync_all ? range::all : is_forward == is_fill ? range::plus : range::minus;
+                        static_assert(!sync_all || std::is_same_v<meta::first<CurInterval>, meta::second<CurInterval>>,
+                            "offset_limit too small");
 
-                        static constexpr check check_v =
-                            minus == plus || PlhInfo::is_tmp_t::value
-                                ? check::none
-                                : close_to_first ? check::lo : close_to_last ? check::hi : check::none;
+                        static constexpr range range_v = minus == plus           ? range::minus
+                                                         : sync_all              ? range::all
+                                                         : is_forward == is_fill ? range::plus
+                                                                                 : range::minus;
+
+                        static constexpr check check_v = minus == plus || PlhInfo::is_tmp_t::value ? check::none
+                                                         : close_to_first                          ? check::lo
+                                                         : close_to_last                           ? check::hi
+                                                                                                   : check::none;
 
                         using type = sync_fun<PlhInfo, range_v, check_v>;
                     };
@@ -306,13 +325,13 @@ namespace gridtools {
 
                     template <class Plh, class DataStores>
                     auto make_data_store(bound<Plh, check::lo>, DataStores const &data_stores) {
-                        return make_global_parameter(
+                        return global_parameter(
                             sid::get_lower_bound<dim::k>(sid::get_lower_bounds(at_key<Plh>(data_stores))));
                     }
 
                     template <class Plh, class DataStores>
                     auto make_data_store(bound<Plh, check::hi>, DataStores const &data_stores) {
-                        return make_global_parameter(
+                        return global_parameter(
                             sid::get_upper_bound<dim::k>(sid::get_upper_bounds(at_key<Plh>(data_stores))));
                     }
 
@@ -324,7 +343,7 @@ namespace gridtools {
                     template <class DataStore>
                     struct is_missing_f {
                         template <class Plh>
-                        using apply = negation<has_key<DataStore, Plh>>;
+                        using apply = std::negation<has_key<DataStore, Plh>>;
                     };
 
                     template <class PlhMap, class DataStores>
@@ -336,9 +355,49 @@ namespace gridtools {
                             hymap::from_keys_values<plhs_t, plhs_t>());
                         return hymap::concat(std::move(data_stores), std::move(extra));
                     }
+
+                    template <class Interval, int Lim = Interval::offset_limit>
+                    using inner_interval = be_api::interval<be_api::level<meta::first<Interval>::splitter, Lim, Lim>,
+                        be_api::level<meta::second<Interval>::splitter, -Lim, Lim>>;
+
+                    template <class Spec, class Grid, class DataStores>
+                    bool validate_k_bounds(Grid const &grid, DataStores const &data_stores) {
+                        for_each<be_api::make_fused_view<Spec>>([&](auto mss) {
+                            using mss_t = decltype(mss);
+                            using interval_t = inner_interval<typename mss_t::interval_t>;
+                            using is_backward_t = be_api::is_backward<typename mss_t::execution_t>;
+                            using plh_map_t =
+                                meta::filter<meta::not_<be_api::get_is_tmp>::apply, typename mss_t::plh_map_t>;
+                            using fill_plhs_t = meta::filter<has_policy_f<cache_io_policy::fill>::apply, plh_map_t>;
+                            using flush_plhs_t = meta::filter<has_policy_f<cache_io_policy::flush>::apply, plh_map_t>;
+                            // Those asserts can trigger even if the user obeys the contract that the data is
+                            // valid within computation area.
+                            // Namely it can happen when k-cache windows are too big for the chosen offset limit.
+                            for_each<meta::if_<is_backward_t, fill_plhs_t, flush_plhs_t>>(
+                                [unchecked_area_begin = grid.k_start(interval_t()), &data_stores](auto info) {
+                                    using plh_info_t = decltype(info);
+                                    constexpr auto extent = plh_info_t::extent_t::kminus::value;
+                                    auto lower_bound = sid::get_lower_bound<dim::k>(
+                                        sid::get_lower_bounds(at_key<typename plh_info_t::plh_t>(data_stores)));
+                                    assert(lower_bound <= unchecked_area_begin + extent);
+                                });
+                            for_each<meta::if_<is_backward_t, flush_plhs_t, fill_plhs_t>>(
+                                [unchecked_area_end = grid.k_start(interval_t()) + grid.k_size(interval_t()),
+                                    &data_stores](auto info) {
+                                    using plh_info_t = decltype(info);
+                                    constexpr auto extent = plh_info_t::extent_t::kplus::value;
+                                    auto upper_bound = sid::get_upper_bound<dim::k>(
+                                        sid::get_upper_bounds(at_key<typename plh_info_t::plh_t>(data_stores)));
+                                    assert(upper_bound >= unchecked_area_end + extent);
+                                });
+                        });
+                        return true;
+                    } // namespace impl_
+
                 } // namespace impl_
                 using impl_::transform_data_stores;
                 using impl_::transform_spec;
+                using impl_::validate_k_bounds;
             } // namespace fill_flush
         }     // namespace gpu_backend
     }         // namespace stencil
