@@ -4,6 +4,9 @@ export GTCMAKE_Boost_NO_BOOST_CMAKE=ON
 export GTCMAKE_Boost_NO_SYSTEM_PATHS=ON
 export GTCMAKE_GT_TESTS_REQUIRE_FORTRAN_COMPILER=ON
 export GTCMAKE_GT_TESTS_REQUIRE_C_COMPILER=ON
+export GTCMAKE_GT_TESTS_REQUIRE_OpenMP=ON
+export GTCMAKE_GT_TESTS_REQUIRE_GPU=ON
+export GT_ENABLE_STENCIL_DUMP=ON
 export GTCMAKE_CMAKE_EXPORT_NO_PACKAGE_REGISTRY=ON
 
 export GTRUN_SBATCH_TIME='00:15:00'
