@@ -7,11 +7,15 @@ label=${label%%-*}
 
 envfile=./jenkins/envs/${label}_$env.sh
 
+export GT_PERFORMANCE_HISTORY_PATH_PREFIX=/apps/${label}/SSL/gridtools/jenkins/gridtools-performance-history-gt2
+
 # use the machines python virtualenv with required modules installed
 if [[ $label = ault ]]; then
     venv_dir=/users/fthaler/public/jenkins/gridtools-venv
+elif [[ $label = dom ]]; then
+    venv_dir=/apps/daint/SSL/gridtools/jenkins/venv
 else
-    venv_dir=/project/c14/jenkins/python-venvs/$label
+    venv_dir=/apps/$label/SSL/gridtools/jenkins/venv
 fi
 source $venv_dir/bin/activate
 
@@ -20,7 +24,7 @@ if [[ $label != "tsa" ]]; then
     export SBATCH_ACCOUNT=d75
 fi
 
-if [[ ! -v build_examples ]] && [[ $env =~ ^(cray|hip)$ ]]; then
+if [[ ! -v build_examples ]] && [[ $env =~ ^(hip)$ ]]; then
     build_examples=false
 fi
 
