@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -31,7 +31,7 @@ struct copy {
 
 template <class T>
 auto grid(T const &) {
-    static_assert(std::rank<T>() == 3, "");
+    static_assert(std::rank<T>() == 3);
     return make_grid(std::extent<T, 0>(), std::extent<T, 1>(), std::extent<T, 2>());
 }
 
