@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -30,16 +30,14 @@ TEST(array, iterate_empty) {
     }
 }
 
-#if !defined(__NVCC__)
 namespace constexpr_compare {
     constexpr array<uint_t, 3> a{0, 0, 0};
     constexpr array<uint_t, 3> b{0, 0, 0};
     constexpr array<uint_t, 3> c{0, 0, 1};
 
-    static_assert(a == b, "");
-    static_assert(a != c, "");
+    static_assert(a == b);
+    static_assert(a != c);
 } // namespace constexpr_compare
-#endif
 
 TEST(array, iterate) {
     const int N = 5;
