@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -14,13 +14,11 @@
 
 #include <gridtools/common/integral_constant.hpp>
 #include <gridtools/common/tuple.hpp>
-#include <gridtools/common/tuple_util.hpp>
 #include <gridtools/sid/concept.hpp>
 
 namespace gridtools {
     namespace {
         using namespace literals;
-        namespace tu = tuple_util;
 
         TEST(multi_shift, smoke) {
             double data[15][42];
@@ -28,13 +26,13 @@ namespace gridtools {
             auto ptr = sid::get_origin(data)();
             auto strides = sid::get_strides(data);
 
-            sid::multi_shift(ptr, strides, tu::make<tuple>(3_c, 5_c, 2_c));
+            sid::multi_shift(ptr, strides, tuple(3_c, 5_c, 2_c));
             EXPECT_EQ(&data[3][5], ptr);
 
-            sid::multi_shift(ptr, strides, tu::make<tuple>(0_c, -2_c));
+            sid::multi_shift(ptr, strides, tuple(0_c, -2_c));
             EXPECT_EQ(&data[3][3], ptr);
 
-            sid::multi_shift(ptr, strides, tu::make<tuple>(-2));
+            sid::multi_shift(ptr, strides, tuple(-2));
             EXPECT_EQ(&data[1][3], ptr);
         }
     } // namespace
