@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2021, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -20,7 +20,7 @@ namespace gridtools {
             struct d;
             using testee_t = positional<d>;
 
-            static_assert(is_sid<testee_t>(), "");
+            static_assert(is_sid<testee_t>());
 
             TEST(positional, smoke) {
                 testee_t testee{1};
